	function get_sp_line( lam_min, lam_max,
     >                        ielem_ion, lambda, lggf, chi )
C ***
C ******************************************************************************
C ***
C ***	Reads Kurucz-Peytremann data from random access file and returns
C ***	all lines (which routines QUATI etc. know about) within given
C ***	wavelength interval, one line per call.  Retrieve all lines by
C ***	repeated calling of this routine.
C ***
C ***	Parameters:
C ***
C ***	LAM_MIN	    > R	  lower limit of wavelength interval to search
C ***	LAM_MAX	    > R	  upper limit
C ***	IELEM_ION   < I	  element code of line, as in 2600 for Fe I
C ***	LAMBDA	    < R	  wavelength of line found
C ***	LGGF	    < R	  log gf of line found
C ***	CHI	    < R	  excitation potential of line in eV
C ***
C ***	GET_SP_LINE < L	  .TRUE.  if line found
C ***			  .FALSE. if finished with wavelength interval
C ***
C ***	Files:
C ***
C ***	Input:	KP01,	KP-List as random access file
C ***
C ***	KP01 format:
C ***
C ***   Binary list contains variables as indicated below; element, ion,
C ***	Jlow, and JUP are written as ONE integer only. The field marked as
C ***	`unknown' is discarded from the binary version as it is always blank.
C ***
C ***                                         element   ion
C                                                  \\   | Jlow
C                                                   \\  | || Jup
C  lambda   log gf  chi low     chi up       unknown \\ | //// 
C                                                     \\|////Design.low Des. up
CxxxxxxxxxxyyyyyyyxxxxxxxxxxxxyyyyyyyyyyyyxxxxxxxxxxxxyyxYYXXYYYYYYYYYYXXXXXXXXXX
C12345678901234567123456789012123456789012123456789012121121212345678901234567890
C
C REAL     REAL     INTEGER     INTEGER     Discarded INTEGERCHARACTER CHARACTER
C
C    5.2682  -.870       0.000 1898181.542             530000  NB                
C    6.0313  -.215       0.000 1658017.343             530000  NB                
C    6.3177 -2.360       0.000 1582860.000            1550002          S2P5S N=5 
C    6.3450 -2.060       0.000 1576040.000            1550002          S2P5S N=5 
C    6.6796  -.790       0.000 1497100.000            1450305          S2P4D N=4 
C    6.8167  -.800       0.000 1466990.000            1350406S2P4                
C    6.8223 -1.330       0.000 1465780.000            1350404S2P4                
C    6.8351 -1.810    2736.000 1465780.000            1350204S2P4                
C    6.9121 -1.940       0.000 1446740.000            1550002          S2P5S N=4 
C ***
C ***
C ***
C ***	Modifications:
C ***
C ***	17-DEC-1992 16:44:29.00		M. Lemke / UT Austin.  Added call to
C ***                                   getenv for Unix to translate logical
C ***                                   name KP.
C ***	 4-NOV-1994 16:16:19.00		ML / IoA.  Fixed bug when checking
C ***					wavelength limits. Used ioff instead
C ***					of off. Set stderr = 0.
C ***	 5-MAY-1995 14:03:47.00		ML / IoA.  Created by stealing
C ***					kp_line.for and converted to function
C ***	 5-MAY-1995 19:56:11.00		ML / IoA.  Fixed problem if lambda
C ***					in first interval of list
C ***	 7-MAY-1996 14:13:35.55		ML / Bamberg.  Adapted record lengths
C ***                                   to longwords for VMS, request explicit
C ***                                   REAL*4 and INTEGER*4 for KP file data
C ***   29-NOV-1999 16:30:05.00		ML / McA.  Open readonly etc. commented.
C ***					recl changed (hopefully correctly).
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	logical get_sp_line
	real lam_min, lam_max
	integer ielem_ion
	real lambda, lggf, chi

C ***
C ***	Local variables
C ***              
	integer nelem, nex_elem, maxpoints
	parameter( nelem = 400, nex_elem = 61, maxpoints = 265587 )
	integer kp
	parameter( kp = 27 )
C ***
C ***	Elements not included in M100.ABU
C ***
	integer exclude_elem( nex_elem )
	data exclude_elem /4,  5,  9, 17, 31, 32, 33, 34, 35, 36, 37,
     >    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 
     >    59, 61, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 75, 77, 78, 
     >    79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 91, 92, 93, 94, 
     >    95, 96, 97, 98, 99/
C ***
C ***	Wavelength points in KP list, for lambda > n100 :
C ***	irec = ioff( mod( lam, 100. ) )
C ***
	integer ioff(0:101)
	data ioff/ 1, 1, 610, 19565, 57711, 98407, 130701, 157031,
     >    174498,
     >	  186308, 194029, 200217, 206081, 210677, 214768, 218098,
     >	  220846, 223338, 225300, 227587, 229556, 231946, 233687,
     >	  235300, 236709, 238185, 239166, 240331, 241277, 242439,
     >	  243253, 244335, 245149, 246132, 246846, 247496, 248064,
     >	  248678, 249214, 249779, 250529, 251095, 251677, 252306,
     >	  252822, 253193, 253927, 254465, 254941, 255377, 255621,
     >	  255856, 256294, 256679, 256910, 257226, 257465, 257615,
     >	  257810, 258130, 258480, 258758, 259045, 259073, 259389,
     >	  259662, 259748, 260120, 260319, 260496, 260705, 260837,
     >	  261069, 261345, 261415, 261721, 261985, 262333, 262471,
     >	  262677, 262821, 262946, 263031, 263232, 263301, 263500,
     >	  263612, 263735, 263865, 264025, 264106, 264240, 264329,
     >	  264526, 264662, 264814, 264929, 265000, 265085, 265300,
     >	  265329, 265587 /
	real off(101)
	data off /5.268200, 10.01490, 100.0023, 200.0004, 300.0052,
     >	  400.0019, 500.0117, 600.0051, 700.0169, 800.0277, 900.0131,
     >	  1000.029, 1095.021, 1200.021, 1300.007, 1400.010, 1500.000,
     >	  1590.016, 1700.051, 1800.039, 1900.015, 2000.025, 2100.064,
     >	  2185.072, 2300.058, 2390.028, 2500.093, 2590.054, 2700.058,
     >	  2790.019, 2900.058, 3000.056, 3100.026, 3200.089, 3300.083,
     >	  3390.055, 3495.098, 3590.011, 3690.052, 3800.002, 3900.005,
     >	  4000.077, 4100.066, 4200.004, 4295.100, 4470.073, 4595.096,
     >	  4690.033, 4800.059, 4875.007, 4950.082, 5100.046, 5200.075,
     >	  5285.005, 5400.009, 5485.051, 5545.015, 5615.058, 5760.093,
     >	  5900.001, 5975.085, 6100.061, 6110.057, 6245.049, 6370.003,
     >	  6415.006, 6600.055, 6690.049, 6785.082, 6895.024, 6960.000,
     >	  7065.012, 7200.054, 7235.070, 7395.068, 7495.085, 7600.011,
     >	  7670.063, 7790.080, 7890.011, 7985.043, 8035.025, 8170.064,
     >	  8235.015, 8390.072, 8490.038, 8570.097, 8685.051, 8780.052,
     >	  8815.084, 8935.041, 9015.081, 9185.041, 9290.025, 9395.034,
     >	  9475.012, 9521.486, 9595.050, 9741.369, 9760.009, 9997.274/


	logical first, done
!	integer   ichi_low, ichi_up, ikey
	integer*4 ichi_low, ichi_up, ikey
	integer i, irec, irec_a, irec_b, irec_old, ielem, ion
	real lam_min_nm, lam_max_nm, old_lam_min_nm, old_lam_max_nm,
     >       chi_low, chi_up

!	real   lam_read, lggf_read
	real*4 lam_read, lggf_read

	save first, old_lam_min_nm, old_lam_max_nm, irec, lam_read

	data first, old_lam_min_nm, old_lam_max_nm /.true., -1., -1./

C ***
C ***	Code
C ***
	if( first ) then
	   open( kp, file = 'KP01', 	!readonly,
     >	         status = 'old', access = 'direct',
     >           form = 'unformatted',
*     >           shared, readonly,
*     >           recl = 5 + 20/4 )
     >	         recl = 5*4 + 20 )
!     >           recl = 5*8 + 20 )
	   first = .false.
	end if

	lam_min_nm = lam_min/10.
	lam_max_nm = lam_max/10.

	if( lam_min_nm .gt. off(101) .or.
     >	    lam_max_nm .lt. off(1)   .or.
     >	    lam_max_nm .le. lam_min_nm ) then

           old_lam_min_nm = -1.
	   old_lam_max_nm = -1.
	   get_sp_line = .false.
	   return

	end if

	if( lam_min_nm .ne. old_lam_min_nm .or.
     >      lam_max_nm .ne. old_lam_max_nm ) then
C ***
C ***	   Find starting point in kp list
C ***
	   i = 1
	   do while( i .le. 100 .and. lam_min_nm .gt. off(i) )
	      i = i + 1	   
	   end do
	   irec_a = ioff( i - 1 )
	   irec_b = ioff( i )
	   irec = irec_a + (irec_b - irec_a)/2
	   done = .false.
	   do while( .not.done )	
              read( kp, rec = irec ) lam_read
	      if( lam_read .lt. lam_min_nm ) then
	         irec_a = irec
	      else if( lam_read .gt. lam_min_nm ) then
	        irec_b = irec
	      else
	         done = .true.
	      end if
              irec_old = irec
	      irec = (irec_a + irec_b)/2
	      if( abs( irec_old - irec ) .le. 1 ) done = .true.
	   end do
           read( kp, rec = irec ) lam_read
	   if( lam_read .lt. lam_min_nm ) irec = irec + 1

	   lam_read = 0.
           irec = irec - 1
	end if

1	continue

	irec = irec + 1
	read( kp, rec = irec ) lam_read, lggf_read, ichi_low, ichi_up,
     >                         ikey
	lggf = lggf_read

	if( (lam_max_nm .ge. lam_read) .and. 
     >	    (irec       .lt. maxpoints)      ) then
C ***
C ***	   decode KP data  
C ***

*	   j_low = (ikey - 10000*(ikey/10000))/100
*	   j_up = ikey - 100*(ikey/100)
	   ielem = ikey/100000
	   ion = ikey/10000 - ielem*10
	   chi_low = ichi_low/1000.
	   chi_up = ichi_up/1000.

	   ielem_ion = ielem*100 + ion
C ***
C ***	   Exclude lines not contained in standard ABU.DAT
C ***
	   if( ion .gt. 4 ) go to 1	! exclude ions > V
	   do i = 1, nex_elem
	      if( exclude_elem(i) .eq. ielem ) go to 1
	   end do
C ***
C ***	   return line found and remember stuff for next one
C ***
	   lambda = lam_read*10.
	   chi = min( chi_low, chi_up )/8065.465

           old_lam_min_nm = lam_min_nm
           old_lam_max_nm = lam_max_nm
	   get_sp_line = .true.
	else
C ***
C ***	   Interval end
C ***
           old_lam_min_nm = -1. ! if called again with same lammin/max,
           old_lam_max_nm = -1.	! start over -- in case two NLTE lines 
				! have same wavelength
	   get_sp_line = .false.
	end if
	end
