        subroutine lineop( dlam, ion, lambda, lggf, exc,
     >                     lgc4, lgc6, gamr, chi, nd )
C ***
C ******************************************************************************
C ***
C ***   Calculate line opacity [cm^2/cm^3] at wavelength dlam from line
C ***   center for given line and for all depth points.
C ***
C ***   Or if called as:
C ***
C ***        entry lcoeff( dlam, ion, lambda, lggf, exc,
C ***                      lgc4, lgc6, gamr, prof, chi, nd )
C ***
C ***   Calculate line absorption coefficent at wavelength dlam from line
C ***   center for given line and for all depth points:
C ***
C ***   chi = sqrt(pi)e^2/m_e/c^2 gf lambda H( alpha, dlam/(v_D*lambda) )
C ***   prof = H( alpha, dlam/(v_D*lambda) )
C ***	norm = lambda/c/v_D/sqrt pi
C ***
C ***
C ***
C ***   dlam      > r   Wavelength from line center []
C ***   ion       > i   Element code, e.g., Fe I: 2600
C ***   lambda    > r   Wavelength of line
C ***   lggf      > r   f-value
C ***   exc       > r   Excitation potential [eV]
C ***                   LCOEFF: used only if LGC4 .EQ. 0
C ***   lgc4      > r   -log C4, if -1: no broadening, 0: Griem/Cowley approx.
C ***   lgc6      > r   -log C6
C ***   gamr      > r   gamma rad, if -1: classical approx.
C ***
C ***   prof(nd)  < r   Voigt function (H) value for
C ***                   depths 1...nd and current dlam
C ***                   (LCOEFF only)
C ***	norm(nd)  < r	Normalization factor to make integral over
C ***			frequency of Voigt function equal to unity
C ***                   (LCOEFF only)
C ***   chi(nd)   < r   LINEOP: line opacity for depths 1...nd
C ***                   LCOEFF: line absorption coefficient (see above)
C ***   nd        > i   number of depth points
C ***
C ***   Input via common blocks:
C ***
C ***   /POP/:          AVEMAS ZETA IONPOT IZP
C ***   /MODEL/:        TEMP PELG XI PGLG
C ***   /LINE/:         HE
C ***   /J/:            WTJ JIZ
C ***
C ***   Output via common blocks:
C ***
C ***   none
C ***
C ***
C ***   This routine requires IONDIS has been called.  Well, ATMOS actually,
C ***   which fills in all those common blocks.
C ***
C ***   This split via entry is more complicated than one would wish but
C ***   I wanted the ability to access all those returned values without
C ***   computing the Voigt function and other bits more often than
C ***   necessary and without having the user to provide all sorts of
C ***   logically unrelated `work' arrays.  The two functions may be
C ***   called in any sequence for the same arguments.
C ***
C ***
C ***   Modifications:
C ***
C ***   14-MAY-1995 13:56:05.00         ML / IoA.  Created.
C ***   17-MAY-1995 19:27:25.00         ML / IoA.  Created from various
C ***                                   intermediate nonsense.
C ***	24-MAY-1995 21:42:01.00		ML / IoA.  Call Voigt function
C ***					with abs(dlam).  H gives wrong
C ***					values if alpha.ne.0 and v<0
C ***	24-MAY-1995 21:58:40.00		ML / IoA.  Return norm also with
C ***					lcoeff.  Changed meaning of prof
C ***	26-APR-1996 19:24:41.10		ML / Bamberg.  OPALION.INC
C ***
C ******************************************************************************
C ***
        implicit none

C ***
C ***   Arguments
C ***
        real dlam
        integer ion, nd
        real lambda, lggf, exc, lgc4, lgc6, gamr
        real chi(nd)            	  ! LINEOP
        real chil(nd), prof(nd), norm(nd) ! LCOEFF

C ***
C ***   Global variables
C ***
        include 'MOD_SIZ.INC'
        include 'POP.INC'
        include 'MODEL.INC'
        include 'LINE.INC'
C ***
C ***   COMMON FOR SUBROUTINE IONDIS
C ***
	include 'OPALION.INC'

C ***
C ***   Some physics...
C ***
        real pi, sqrt_pi, r_gas, m_e, e, c, hplanck, k, c2
        parameter( pi = 3.14159265358979323846264,
     >             sqrt_pi = 1.77245385090551602729816,
     >             r_gas = 8.31451e7,   ! gas constant
     >             m_e = 9.10939e-28,   ! electron mass
     >             e = 4.803207e-10,    ! electron charge
     >             c = 2.99792458e10,   ! speed of light
     >             hplanck = 6.626076e-27,      ! Planck
     >             k = 1.38066e-16,     ! Boltzmann
     >             c2 = 1.43877e0       ! h/k*c, for (h nu/kT)
     >           )
        real const
        parameter( const = pi*e*e/m_e/c )

C ***
C ***   Local variables
C ***
        logical force_dlam, e_lineop, e_lcoeff
        integer id, stage, elem
        real n_i(m_depth), lprof(m_depth)
        real v_d(m_depth), lnorm(m_depth), alpha(m_depth)
        real gamma_h, gamma_e, gamma_r, lg_t, sig0
        real domega_d, weight, stimem, x


        integer lold_ion, lold_nd
        real lold_lambda, lold_exc

        integer old_ion, old_nd
        real old_lambda, old_lggf, old_exc, old_lgc4, old_lgc6,
     >       old_gamr, old_dlam

C ***
C ***   Functions
C ***
        real exp10, h, gam4def


        save lold_ion, lold_nd, lold_lambda, lold_exc
        save old_ion, old_nd, old_lambda, old_lggf, old_exc, old_lgc4,
     >       old_lgc6, old_gamr, old_dlam
        save n_i, v_d, lnorm, alpha, lprof, sig0

        data lold_ion, lold_nd, lold_lambda, lold_exc
     >      /-1,      -1,     -1.,        -1. /
        data old_ion, old_nd, old_lambda, old_lggf, old_exc, old_lgc4,
     >       old_lgc6, old_gamr, old_dlam /
     >       -1,      -1,     -1.,        -99.,     -1.,     -2.,
     >       -1.,      -2.,      -1.e+38 /

        exp10(x) = exp( x*2.3025851 )

C ***
C ***   Code
C ***
        if( ion .ne. lold_ion .or.
     >      lambda .ne. lold_lambda .or.
     >      exc .ne. lold_exc .or.
     >      nd .gt. lold_nd ) then

C ***
C ***      New line.  Compute wavelength independent bits.
C ***
           elem = ion/100
           stage = mod( ion, 100 )

           do id = 1, nd
              stimem = 1. - exp( -c2*1e8/(temp(id)*lambda) )

              n_i(id) = stimem*
     >                  exp10( -5040.*exc/temp(id) )*
     >                  zeta(id,stage,izp(elem))/avemas

           end do
        end if

C ***
C ***   Book keeping lineop...
C ***
        lold_ion = ion
        lold_lambda = lambda
        lold_exc = exc
        lold_nd = nd

        e_lcoeff = .false.
        e_lineop = .true.
        go to 1



        entry lcoeff( dlam, ion, lambda, lggf, exc,
C ***         ======
     >                lgc4, lgc6, gamr, prof, norm, chil, nd )

        e_lcoeff = .true.
        e_lineop = .false.
1       continue


        force_dlam = .false.
        if( ion .ne. old_ion .or.
     >      lambda .ne. old_lambda .or.
     >      lggf .ne. old_lggf .or.
     >      exc .ne. old_exc .or.
     >      lgc4 .ne. old_lgc4 .or.
     >      lgc6 .ne. old_lgc6 .or.
     >      gamr .ne. old_gamr .or.
     >      nd .gt. old_nd ) then

C ***
C ***      New line.  Compute wavelength independent bits.
C ***
           elem = ion/100
           stage = mod( ion, 100 )

           if( ion .ne. old_ion ) then
C ***
C ***         Doppler velocity in units of c
C ***
              weight = wtj( jiz(elem) )
              x = 2.*r_gas/weight 
              do id = 1, nd
                 v_d(id) = sqrt( xi(id)*xi(id)*1.e10 + x*temp(id) )/c
              end do                           ! (km -> cm)^2
           end if

C ***
C ***      l_line (lnorm) without Voigt function,
C ***      alpha  for Voigt function
C ***

           do id = 1, nd
              lnorm(id) = 1e-8*lambda/v_d(id)/c/sqrt_pi
           end do         ! help (?) the vectorizer...
           do id = 1, nd  ! function call below: gam4def

              lg_t = log10( temp(id) )

              if( lgc6 .le. 0. ) lgc6 = 32.  ! drrca ~= pi a_0^2
              gamma_h = exp10( -0.4*lgc6 + 11.565131 + pglg(id) -
     >                         0.7*lg_t )*
     >                  zetah(id)*( 1. + he/2.4194 )

              if( lgc4 .gt. 0. ) then
                 gamma_e = exp10( -2./3.*lgc4 + pelg(id) - 5./6.*lg_t +
     >                            10.84677 )
              else if( lgc4 .eq. 0. ) then
                 gamma_e = gam4def( stage+1, ionpot(stage,izp(elem)),
     >                              exc, 0. ) +
     >                     gam4def( stage+1, ionpot(stage,izp(elem)),
     >                              exc+12398.54/lambda, 0. )
                 gamma_e = gamma_e*exp10( pelg(id) - lg_t )/k
              else
                 gamma_e = 0.
              end if

              if( gamr .lt. 0. ) then
                 gamma_r = 2.22e7/lambda/lambda
              else
                 gamma_r = gamr
              end if

              domega_d = ( 4.*pi*c*v_d(id) )/lambda    ! 2*\Delta\omega_D
              alpha(id) = (gamma_h + gamma_e + gamma_r)/domega_d
           end do

           sig0 = const*exp10(lggf)

           force_dlam = .true.
        end if


C ***
C ***   New wavelength point...
C ***
        if( abs(dlam) .ne. old_dlam .or. force_dlam ) then
           do id = 1, nd
              lprof(id) = h( alpha(id), abs(dlam)/(v_d(id)*lambda) )
           end do
        end if

        if( e_lcoeff ) then
           do id = 1, nd
              prof(id) = lprof(id)
              norm(id) = lnorm(id)
              chil(id) = sig0*lprof(id)*lnorm(id)
           end do
        end if
        if( e_lineop ) then
           do id = 1, nd
              chi(id) = sig0*lprof(id)*lnorm(id)*n_i(id)
           end do
        end if
C ***
C ***   Book keeping...
C ***
        old_ion = ion
        old_lambda = lambda
        old_lggf = lggf
        old_exc = exc
        old_lgc4 = lgc4
        old_lgc6 = lgc6
        old_gamr = gamr
        old_nd = nd
        old_dlam = abs(dlam)

        return

C ***
C ***   Some recreational maths, no code following
C ***

*            lg_theta = log10( 5040./temp(id) )
*        0.7*lg_theta = 0.7*log10( 5040. ) - 0.7*log10( temp(id) )
*                     = 0.7*3.7024305 - 0.7*log10( temp(id) )
*                     = 2.5917014     - 0.7*log10( temp(id) )

*             p_h = 2.*zetah(id)*exp10( pglg(id) )

*             gamma_h = exp10( -0.4*lgc6 + 8.6724 + 0.7*lg_theta )*
*    >                  p_h*( 1. + he/2.4194 )
*
*                     = exp10( -0.4*lgc6 + 8.6724 + 2.5917014 -
*    >                         0.7*log10(temp(id)) )*
*    >                  p_h*( 1. + he/2.4194 )

*                     = exp10( -0.4*lgc6 + 11.264101 - 
*    >                         0.7*log10(temp(id)) )*
*    >                  p_h*( 1. + he/2.4194 )

*                     = exp10( -0.4*lgc6 + 11.264101 -
*    >                         0.7*log10(temp(id)) )*
*    >                  2.*zetah(id)*exp10( pelg(id) )*( 1. + he/2.4194 )

*                     = exp10( -0.4*lgc6 + 11.264101 + 0.3010 + pglg(id) -
*    >                         0.7*log10(temp(id)) )*
*    >                  zetah(id)*( 1. + he/2.4194 )

*                     = exp10( -0.4*lgc6 + 11.565131 + pglg(id) -
*    >                         0.7*log10(temp(id)) )*
*    >                  zetah(id)*( 1. + he/2.4194 )

*             gamma_e = exp10( -2./3.*lgc4 + pelg(id) + 5./6.*lg_theta +
*    >                         7.76140961 )
*                     = exp10( -2./3.*lgc4 + pelg(id) - 5./6.*lg_t +
*    >                         7.76140961 + 5./6.*log10(5040.) )
*                     = exp10( -2./3.*lgc4 + pelg(id) - 5./6.*lg_t +
*    >                         7.76140961 + 3.0853588 )

        end
